// code by jph
package mid;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.Receiver;
import javax.sound.midi.Transmitter;

public final class MidiIn extends Midi {
  public static final MidiIn instance = new MidiIn();

  private MidiIn() {
  }

  @Override
  protected boolean acceptMidiDevice(MidiDevice.Info myMidiDeviceInfo) throws Exception {
    // System.out.println("here");
    return hasTransmitter(MidiSystem.getMidiDevice(myMidiDeviceInfo));
  }

  static boolean hasTransmitter(MidiDevice myMidiDevice) {
    int rec = myMidiDevice.getMaxTransmitters();
    return rec == -1 || 0 < rec; // -1 if an unlimited number of connections is available
  }

  private Transmitter mySingletonTransmitter = null;

  private Transmitter getNewTransmitter(MidiDevice myMidiDevice) throws Exception {
    resetTransmitter();
    // MidiDevice myMidiDevice = getMidiDevice();
    // if (myMidiDevice != null)
    {
      if (hasTransmitter(myMidiDevice)) {
        // technically this check is not proof that the device is really already in use
        // if (new FileBlock(MidiIn.class.getSimpleName()).isActive())
        // throw new Exception("midi transmitter is system static and already in use");
        if (!myMidiDevice.isOpen()) // first call returns false, once opened call always returns true
          myMidiDevice.open();
        mySingletonTransmitter = myMidiDevice.getTransmitter();
        Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
          @Override
          public void run() {
            System.out.println("ShutdownHook MidiIn getTransmitter");
            resetTransmitter();
          }
        }));
        return mySingletonTransmitter;
      }
    }
    throw new Exception("device has no transmitter");
  }

  public void linkTransmitterWith(Transmitter myTransmitter, Receiver myReceiver) throws Exception {
    // Transmitter myTransmitter = getNewTransmitter();
    if (myTransmitter != mySingletonTransmitter)
      System.out.println("myTransmitter != mySingletonTransmitter");
    if (myTransmitter.getReceiver() != null) {
      System.out.println("[ YOU SHOULD NEVER SEE THIS ]");
      myTransmitter.getReceiver().close();
    }
    myTransmitter.setReceiver(myReceiver);
  }

  public void resetTransmitter() {
    if (mySingletonTransmitter != null) {
      Receiver myReceiver = mySingletonTransmitter.getReceiver();
      if (myReceiver != null) {
        System.out.println("myReceiver.close()");
        myReceiver.close();
      }
      mySingletonTransmitter.close();
      mySingletonTransmitter = null;
    }
  }
  // @Override
  // protected AutoCloseable getAutoCloseable() throws Exception {
  // MidiDevice myMidiDevice = getMidiDevice();
  // if (hasTransmitter(myMidiDevice))
  // return myMidiDevice.getTransmitter();
  // throw new Exception("device has no transmitter");
  // }
}
