// code by jph
package mid;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiMessage;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.Receiver;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;

/** implementation of {@link MidiOutput} */
public class MidiOutputServer implements MidiOutput, AutoCloseable {
  private List<Long> myVector = new LinkedList<>(); // ensures that not more than a certain sequences are played within a short period of time
  Sequencer mySequencer = null;
  Receiver myReceiver = null;
  public final long over = 1000_000_000L; // myManager.getInteger("PlaySequence.Period", 1000) *
  private final MidiDevice myMidiDevice;

  public MidiOutputServer(MidiDevice myMidiDevice) throws Exception {
    this.myMidiDevice = myMidiDevice;
    myReceiver = MidiOut.getReceiver(myMidiDevice);
    mySequencer = MidiSystem.getSequencer(false);
    mySequencer.getTransmitter().setReceiver(myReceiver);
    mySequencer.open();
  }

  public String getDeviceInfo() {
    return myMidiDevice.getDeviceInfo().toString().replaceAll("\\W", "");
  }

  @Override
  public void startSequence(Sequence mySequence) {
    long toc = System.nanoTime();
    Iterator<Long> myIterator = myVector.iterator();
    while (myIterator.hasNext()) {
      long tic = myIterator.next();
      if (over < toc - tic)
        myIterator.remove();
    }
    if (myVector.size() < 3) { // myManager.getInteger("PlaySequence.Count", 3)
      myVector.add(toc);
      // END: mechanism that prevents LoopBe1 from muting
      try {
        mySequencer.setSequence(mySequence);
        mySequencer.start();
      } catch (Exception myException) {
        myException.printStackTrace();
      }
    } else
      System.out.println("prevented midi sequencer over-clocking");
    // new Exception("prevented midi sequencer over-clocking").printStackTrace();
  }

  @Override
  public boolean isRunning() {
    return mySequencer.isRunning();
  }

  @Override
  public long getTickPosition() {
    return mySequencer.getTickPosition();
  }

  @Override
  public void setTempoInBPM(float myFloat) {
    mySequencer.setTempoInBPM(myFloat);
  }

  @Override
  public void stopSequencers() {
    mySequencer.stop();
  }

  @Override
  public void send(MidiMessage myMidiMessage, long ticks) {
    myReceiver.send(myMidiMessage, ticks);
  }

  @Override
  public void close() throws Exception {
    stopSequencers();
    mySequencer.close();
    myReceiver.close();
  }
}
