// code by jph
package mid.gui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiSystem;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;

import mid.MidiIO;
import mid.MidiIn;
import mid.MidiOut;
import sys.CommonFileChooser;
import sys.gui.ComponentFactory;
import sys.gui.StandardMenu;
import tpf.ThePirateFugues;
import tpf.gui.GlobalSwitch;
import tpf.mid.MidiLocal;
import tpf.mid.seq.MidiListings;
import tpf.mid.seq.MidiSafemode;

public class MidiMenu extends StandardMenu {
  public static JButton createJButton() {
    JButton myJButton = ComponentFactory.getJButton("icon.mdq.png", "midi configuration");
    myJButton.setBackground(Color.gray);
    myJButton.addActionListener(myActionEvent -> new MidiMenu().southEast(myJButton));
    return myJButton;
  }

  @Override
  protected void design(JPopupMenu myJPopupMenu) {
    {
      JMenu myJMenu = new JMenu("midi out");
      myJMenu.setToolTipText("device to receive playback for sound synthesis");
      myJMenu.setIcon(ComponentFactory.getIcon("icon.mdp.png"));
      ButtonGroup myButtonGroup = new ButtonGroup();
      for (MidiDevice.Info myMidiDeviceInfo : MidiOut.instance.getMidiDeviceInfoCompact()) {
        JRadioButtonMenuItem myJRadioButtonMenuItem = new JRadioButtonMenuItem(myMidiDeviceInfo.toString());
        myJRadioButtonMenuItem.setSelected( //
            myMidiDeviceInfo.equals(MidiIO.getInstance().myMidiDeviceInfoAutoCloseable_O.getMidiDeviceInfo()));
        myJRadioButtonMenuItem.addActionListener(myActionEvent -> MidiIO.getInstance().setMidiOut(myMidiDeviceInfo));
        
        myJRadioButtonMenuItem.addActionListener(new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent myActionEvent) {
            MidiIO.getInstance().setMidiOut(myMidiDeviceInfo);
          }
        });
        
        myButtonGroup.add(myJRadioButtonMenuItem);
        myJMenu.add(myJRadioButtonMenuItem);
      }
      myJMenu.addSeparator();
      {
        JRadioButtonMenuItem myJRadioButtonMenuItem = new JRadioButtonMenuItem("(none)");
        myJRadioButtonMenuItem.setForeground(new Color(96, 96, 96));
        myJRadioButtonMenuItem.setSelected(!MidiIO.getInstance().myMidiDeviceInfoAutoCloseable_O.isAvailable());
        myJRadioButtonMenuItem.addActionListener(myActionEvent -> MidiIO.getInstance().setMidiOut(null));
        myButtonGroup.add(myJRadioButtonMenuItem);
        myJMenu.add(myJRadioButtonMenuItem);
      }
      myJPopupMenu.add(myJMenu);
    }
    // ---
    {
      JMenu myJMenu = new JMenu("midi in");
      myJMenu.setToolTipText("not relevant for " + ThePirateFugues.class.getSimpleName());
      myJMenu.setIcon(ComponentFactory.getIcon("icon.mdp.png"));
      ButtonGroup myButtonGroup = new ButtonGroup();
      for (MidiDevice.Info myMidiDeviceInfo : MidiIn.instance.getMidiDeviceInfoCompact()) {
        JRadioButtonMenuItem myJRadioButtonMenuItem = new JRadioButtonMenuItem(myMidiDeviceInfo.toString());
        myJRadioButtonMenuItem.setSelected( //
            myMidiDeviceInfo.equals(MidiIO.getInstance().myMidiDeviceInfoAutoCloseable_I.getMidiDeviceInfo()));
        myJRadioButtonMenuItem.addActionListener(myActionEvent -> MidiIO.getInstance().setMidiIn(myMidiDeviceInfo));
        myButtonGroup.add(myJRadioButtonMenuItem);
        myJMenu.add(myJRadioButtonMenuItem);
      }
      myJMenu.addSeparator();
      {
        // myMidiDeviceInfo.equals(MidiIO.getInstance().myMidiDeviceInfoAutoCloseableOut.getMidiDeviceInfo())
        JRadioButtonMenuItem myJRadioButtonMenuItem = new JRadioButtonMenuItem("(none)");
        myJRadioButtonMenuItem.setForeground(new Color(96, 96, 96));
        myJRadioButtonMenuItem.setSelected(!MidiIO.getInstance().myMidiDeviceInfoAutoCloseable_I.isAvailable());
        myJRadioButtonMenuItem.addActionListener(myActionEvent -> MidiIO.getInstance().setMidiIn(null));
        myButtonGroup.add(myJRadioButtonMenuItem);
        myJMenu.add(myJRadioButtonMenuItem);
      }
      myJPopupMenu.add(myJMenu);
      myJMenu.setEnabled(0 < myJMenu.getItemCount());
    }
    myJPopupMenu.addSeparator();
    // ---
    // File[] myFiles = new File("get", "synth").listFiles(new FilenameFilter() {
    // @Override
    // public boolean accept(File myFile, String myString) {
    // return true; // myString.toLowerCase().endsWith(".rns");
    // }
    // });
    // if (myFiles != null && 0 < myFiles.length) {
    // final JMenu myJMenu = new JMenu("synthesizer");
    // myJMenu.setIcon(ComponentFactory.getIcon("icon.pph.png"));
    // for (File myFile : myFiles) {
    // Filename myFilename = new Filename(myFile);
    // JMenuItem myJMenuItem = new JMenuItem(myFilename.myFile.getName(), ComponentFactory.getIcon("icon." + myFilename.myExtension + ".png"));
    // myJMenuItem.addActionListener(myActionEvent -> DesktopHelper.open(myFile));
    // myJMenu.add(myJMenuItem);
    // }
    // myJPopupMenu.add(myJMenu);
    // myJPopupMenu.addSeparator();
    // }
    // ---
    {
      JMenuItem myJMenuItem = new JMenuItem("playback file", ComponentFactory.getIcon("icon.mid.png"));
      myJMenuItem.setToolTipText("send midi sequence of file to midi out device");
      CommonFileChooser myCommonFileChooser = CommonFileChooser.forMidi(myFile -> {
        MidiLocal.getInstance().getMidiOutput().stopSequencers();
        try {
          MidiLocal.getInstance().getMidiOutput().startSequence(MidiSystem.getSequence(myFile));
        } catch (Exception myException) {
          myException.printStackTrace();
        }
      });
      myCommonFileChooser.add(myJMenuItem);
      myJPopupMenu.add(myJMenuItem);
    }
    {
      JMenuItem myJMenuItem = new JMenuItem("playback file (safemode)", ComponentFactory.getIcon("icon.mis.png"));
      myJMenuItem.setToolTipText("without system or sequencer exclusive events");
      CommonFileChooser myCommonFileChooser = CommonFileChooser.forMidi(myFile -> {
        try {
          MidiSafemode.defaultExport(myFile);
          MidiLocal.getInstance().getMidiOutput().stopSequencers();
          MidiLocal.getInstance().getMidiOutput().startSequence(MidiSafemode.getSequence(myFile));
        } catch (Exception myException) {
          myException.printStackTrace();
        }
      });
      // };
      myCommonFileChooser.add(myJMenuItem);
      myJPopupMenu.add(myJMenuItem);
    }
    {
      JMenuItem myJMenuItem = new JMenuItem("file listing", ComponentFactory.getIcon("icon.mil.png"));
      myJMenuItem.setToolTipText("midi event listing as webpage");
      CommonFileChooser myCommonFileChooser = CommonFileChooser.forMidi(myFile -> {
        try {
          GlobalSwitch.open(MidiListings.defaultExportHtml(myFile));
        } catch (Exception myException) {
          myException.printStackTrace();
        }
      });
      // };
      myCommonFileChooser.add(myJMenuItem);
      myJPopupMenu.add(myJMenuItem);
    }
    myJPopupMenu.addSeparator();
    myJPopupMenu.add(new JMenuItem("close menu", ComponentFactory.getIcon("icon.nop.gif")));
  }
}
