// code by jph
package sys;

import java.io.File;
import java.util.Arrays;
import java.util.List;

public final class ProcessManager {
  private final List<ProcessBuilder> myList;

  /** @param myList will not be modified (but contents might) */
  public ProcessManager(List<ProcessBuilder> myList) {
    this.myList = myList;
    // ---
    myList.forEach(this::safe);
  }

  private void safe(ProcessBuilder myProcessBuilder) {
    switch (DesktopHelper.getOperatingSystem()) {
    case Windows:
      List<String> command = myProcessBuilder.command();
      String exec = command.get(0);
      if (!exec.startsWith("\"") && exec.contains(" "))
        command.set(0, '\"' + exec + '\"');
      break;
    default:
      break;
    }
  }

  public ProcessManager(ProcessBuilder myProcessBuilder) {
    this(Arrays.asList(new ProcessBuilder[] { myProcessBuilder }));
  }

  public Process executeSafeWaitFor(File myFile) throws Exception {
    Process myProcess = executeSafe(myFile);
    myProcess.waitFor();
    return myProcess;
  }

  public boolean bat = true;
  public boolean log = true;

  public void setSilent() {
    bat = false;
    log = false;
  }

  public Process executeSafe(File myFile) {
    Filename myFilename = new Filename(myFile);
    File myDirectory = myFile.getParentFile();
    File myBatchFile = myFilename.withExtension("bat");
    if (bat)
      toBatchFile(myBatchFile);
    Process myProcess = null;
    try {
      switch (DesktopHelper.getOperatingSystem()) {
      case Windows: {
        // execute as batch
        toBatchFile(myBatchFile);
        if (!bat)
          myBatchFile.deleteOnExit();
        ProcessBuilder myProcessBuilder = new ProcessBuilder(myBatchFile.getAbsolutePath());
        myProcessBuilder.directory(myDirectory);
        if (log)
          myProcessBuilder.redirectError(myFilename.withExtension("log"));
        myProcess = myProcessBuilder.start();
        break;
      }
      case Linux:
        int count = 0;
        for (ProcessBuilder myProcessBuilder : myList) {
          myProcessBuilder.directory(myDirectory);
          if (log)
            myProcessBuilder.redirectError(myFilename.withExtension("log")); // meaningful for list
                                                                             // of tasks!?
          myProcess = myProcessBuilder.start();
          ++count;
          if (count < myList.size())
            myProcess.waitFor();
        }
        break;
      default:
        break;
      }
    } catch (Exception myException) {
      myException.printStackTrace();
    }
    return myProcess;
  }

  public String toString() {
    StringBuffer myStringBuffer = new StringBuffer();
    for (ProcessBuilder myProcessBuilder : myList) {
      StringBuffer myCommandBuffer = new StringBuffer();
      myProcessBuilder.command().forEach(s -> myCommandBuffer.append(" " + s));
      myStringBuffer.append(myCommandBuffer.substring(1) + '\n');
    }
    return myStringBuffer.toString();
  }

  public void toBatchFile(File myFile) {
    BinaryFile.write(myFile, toString().getBytes());
  }
}
