// code by jph
package sys.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.util.function.BiFunction;

import javax.swing.JLabel;

import sys.col.Hue;

public class ImageConvert {
  final BufferedImage myBufferedImage;
  final Dimension myDimension;

  public ImageConvert(Image myImage) {
    myBufferedImage = toType(myImage, BufferedImage.TYPE_INT_ARGB);
    myDimension = new Dimension(myBufferedImage.getWidth(), myBufferedImage.getHeight());
  }

  public static BufferedImage toType(Image myImage, int type) {
    JLabel myJLabel = new JLabel();
    Dimension myDimension = new Dimension(myImage.getWidth(myJLabel), myImage.getHeight(myJLabel));
    BufferedImage myReturn = new BufferedImage(myDimension.width, myDimension.height, type); // BufferedImage.TYPE_BYTE_GRAY
    myReturn.createGraphics().drawImage(myImage, 0, 0, myJLabel);
    return myReturn;
  }

  public BufferedImage toType(int type) {
    return toType(myBufferedImage, type);
  }

  public final void apply(BiFunction<Point, Integer, Integer> myBiFunction) {
    for (int x = 0; x < myDimension.width; ++x)
      for (int y = 0; y < myDimension.height; ++y)
        myBufferedImage.setRGB(x, y, myBiFunction.apply(new Point(x, y), myBufferedImage.getRGB(x, y)));
  }

  public static BiFunction<Point, Integer, Integer> reduceIntensity(double factor) {
    return (p, rgba) -> {
      int rgb = rgba & 0xffffff;
      int a = (rgba >> 24) & 0xff;
      a = (int) Math.round(a * factor);
      return (a << 24) + rgb;
    };
  }

  public static BiFunction<Point, Integer, Integer> modifHSV(double s, double v) {
    return (p, rgba) -> {
      Hue myHue = Hue.fromColor(new Color(rgba, true));
      return new Hue(myHue.h, myHue.s * s, myHue.v * v, myHue.a).rgba.getRGB();
    };
  }

  public static BiFunction<Point, Integer, Integer> makeWhiteTransparent(double mult) {
    return (p, rgba) -> {
      Hue myHue = Hue.fromColor(new Color(rgba, true));
      int rgb = rgba & 0xffffff;
      int a = (int) Math.min(Math.max(0, (1 + myHue.s - mult * myHue.v) * 255), 255);
      return (a << 24) + rgb;
    };
  }

  public static BufferedImage apply(Image myImage, BiFunction<Point, Integer, Integer> myBiFunction) {
    ImageConvert myImageConvert = new ImageConvert(myImage);
    myImageConvert.apply(myBiFunction);
    return myImageConvert.toType(BufferedImage.TYPE_4BYTE_ABGR);
  }

  public static BufferedImage reduceIntensity(Image myImage, double factor) {
    return apply(myImage, reduceIntensity(factor));
  }

  public static BufferedImage modifHSV(Image myImage, double s, double v) {
    return apply(myImage, modifHSV(s, v));
  }
}
