// code by jph
package sys.gui;

import java.awt.Cursor;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;

/** ResizeCorner is a {@link JLabel} that allows the user to resize the ancestor {@link Window} by mouse drag. */
public class ResizeCorner {
  public static final ImageIcon myImageIcon = ComponentFactory.getIcon("icon32/scale.png");
  public final JLabel myJLabel = new JLabel(myImageIcon);

  public ResizeCorner(final boolean resize_w, final boolean resize_h) {
    myJLabel.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseExited(MouseEvent myMouseEvent) {
        myJLabel.setCursor(Cursor.getDefaultCursor());
      }

      @Override
      public void mouseEntered(MouseEvent myMouseEvent) {
        myJLabel.setCursor(new Cursor(Cursor.HAND_CURSOR));
      }
    });
    myJLabel.addMouseMotionListener(new MouseMotionListener() {
      boolean drag = false;
      int mouse_x;
      int mouse_y;

      @Override
      public void mouseMoved(MouseEvent myMouseEvent) {
        myJLabel.setCursor(new Cursor(Cursor.HAND_CURSOR));
        drag = false;
      }

      @Override
      public void mouseDragged(MouseEvent myMouseEvent) {
        if (!drag) {
          mouse_x = myMouseEvent.getXOnScreen();
          mouse_y = myMouseEvent.getYOnScreen();
          drag = true;
        } else {
          Window myWindow = SwingUtilities.getWindowAncestor(myJLabel);
          myJLabel.setCursor(new Cursor(Cursor.MOVE_CURSOR));
          Rectangle myRectangle = myWindow.getBounds();
          if (resize_w)
            myRectangle.width += myMouseEvent.getXOnScreen() - mouse_x;
          if (resize_h)
            myRectangle.height += myMouseEvent.getYOnScreen() - mouse_y;
          mouse_x = myMouseEvent.getXOnScreen();
          mouse_y = myMouseEvent.getYOnScreen();
          myWindow.setBounds(myRectangle);
        }
      }
    });
  }
}
