// code by jph
package sys.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.swing.JComponent;
import javax.swing.JFrame;

import com.sun.awt.AWTUtilities;

import sys.DesktopHelper;

/** SplashScreen features transparency. SplashScreen is an alternative to {@link java.awt.SplashScreen} */
public class SplashScreen {
  private JFrame myJFrame;
  private BufferedImage myBufferedImage = null;
  private String myString = "";
  private JComponent myJComponent = new JComponent() {
    @Override
    protected void paintComponent(Graphics myGraphics) {
      if (myBufferedImage != null)
        myGraphics.drawImage(myBufferedImage, 0, 0, myJFrame);
      myGraphics.setColor(Color.white);
      myGraphics.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
      int pix = (width - myGraphics.getFontMetrics().stringWidth(myString)) / 2;
      myGraphics.drawString(myString, Math.max(3, pix), height - 14);
    }
  };
  final int width;
  final int height;

  private void setString(String myString) {
    this.myString = myString;
    myJComponent.repaint();
  }

  Handler myHandler = new Handler() {
    @Override
    public synchronized void publish(LogRecord myLogRecord) {
      myString = myLogRecord.getMessage();
      myJComponent.repaint();
    }

    @Override
    public void flush() {
    }

    @Override
    public void close() throws SecurityException {
    }
  };
  Timer myTimer = null;

  private SplashScreen(String myString, File myFile) throws Exception {
    myBufferedImage = ImageIO.read(myFile);
    myJFrame = new JFrame(" "); // non-empty, otherwise icon does not appear centered
    myJFrame.setIconImage(ComponentFactory.getIcon(myString).getImage());
    myJFrame.setUndecorated(true);
    Dimension myScreen = Toolkit.getDefaultToolkit().getScreenSize();
    width = myBufferedImage.getWidth();
    height = myBufferedImage.getHeight();
    myJFrame.setBounds((myScreen.width - width) / 2, (myScreen.height - height) / 2, width, height);
    Container myContainer = myJFrame.getContentPane();
    myContainer.setLayout(new BorderLayout());
    myJComponent.addMouseListener(new MouseAdapter() {
      @Override
      public void mousePressed(MouseEvent myMouseEvent) {
        fastDispose();
      }
    });
    myContainer.add("Center", myJComponent);
    myJFrame.setAlwaysOnTop(true);
    if (!DesktopHelper.isLinux()) {
      myTimer = new Timer();
      myTimer.schedule(new TimerTask() {
        double tim = 0;

        @Override
        public void run() {
          try {
            AWTUtilities.setWindowOpaque(myJFrame, false);
            AWTUtilities.setWindowOpacity(myJFrame, (float) Math.sin(tim));
            tim += 0.02;
            if (Math.PI * 3 / 4 < tim)
              myTimer.cancel();
          } catch (Exception myException) {
            myException.printStackTrace();
            myTimer.cancel();
            System.out.println("terminate splashscreen timer after exception");
          }
        }
      }, 0, 20);
    }
    myJFrame.setLocationRelativeTo(null);
    myJFrame.setVisible(true);
    myHandler.setLevel(Level.ALL);
    Logger.getGlobal().addHandler(myHandler); // it's ok to register to logs here
  }

  private void fastDispose() {
    Logger.getGlobal().removeHandler(myHandler);
    if (myTimer != null)
      myTimer.cancel();
    myJFrame.dispose();
    instance = null;
  }

  public static void hide() {
    if (instance != null)
      instance.fastDispose();
  }

  private static SplashScreen instance = null;

  public static void setText(String myString) {
    if (instance != null)
      instance.setString(myString);
  }

  public static void create(String myString, File myFile) {
    try {
      instance = new SplashScreen(myString, myFile);
    } catch (Exception myException) {
      myException.printStackTrace();
    }
  }

  public static void main(String[] args) {
    SplashScreen.create("icon32/blank.png", new File("get/img/cover/splash_blue.jpg"));
  }
}
