// code by jph
package sys.gui;

import java.awt.Container;
import java.awt.Cursor;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import sys.DesktopHelper;

/** WindowAncestor equips a {@link JComponent} with a {@link MouseListener} that allows the used to
 * drag the window, by dragging on the component.
 * 
 * if the operating system is Linux the listener will not be added since the effect can be achieved
 * with alt+drag */
public class WindowAncestor {
  public static void movesBy(final Container myContainer) {
    if (DesktopHelper.isLinux())
      return;
    myContainer.addMouseListener(new MouseAdapter() {
      @Override
      public void mousePressed(MouseEvent myMouseEvent) {
        myContainer.setCursor(new Cursor(Cursor.MOVE_CURSOR));
      }

      @Override
      public void mouseReleased(MouseEvent myMouseEvent) {
        myContainer.setCursor(Cursor.getDefaultCursor());
      }
    });
    myContainer.addMouseMotionListener(new MouseMotionListener() {
      boolean drag = false;
      int mouse_x;
      int mouse_y;

      @Override
      public void mouseMoved(MouseEvent myMouseEvent) {
        drag = false;
      }

      @Override
      public void mouseDragged(MouseEvent myMouseEvent) {
        if (!drag) {
          mouse_x = myMouseEvent.getXOnScreen();
          mouse_y = myMouseEvent.getYOnScreen();
          drag = true;
        } else {
          Window myWindow = SwingUtilities.getWindowAncestor(myContainer);
          Rectangle myRectangle = myWindow.getBounds();
          myRectangle.x += myMouseEvent.getXOnScreen() - mouse_x;
          myRectangle.y += myMouseEvent.getYOnScreen() - mouse_y;
          mouse_x = myMouseEvent.getXOnScreen();
          mouse_y = myMouseEvent.getYOnScreen();
          myWindow.setBounds(myRectangle);
        }
      }
    });
  }
}
