// code by jph
package sys.mat;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.NavigableSet;
import java.util.TreeSet;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class IntRanges<Type extends IntRange> implements Iterable<Type> {
  private NavigableSet<Type> myNavigableSet = new TreeSet<>();

  private boolean attemptUnion(Type myType, Type myIntRange) {
    if (myType != null && myType.isUnionable(myIntRange)) {
      myNavigableSet.remove(myType);
      union(myType.union(myIntRange));
      return true;
    }
    return false;
  }

  public void union(Type myIntRange) {
    if (attemptUnion(myNavigableSet.lower(myIntRange), myIntRange))
      return;
    if (attemptUnion(myNavigableSet.higher(myIntRange), myIntRange))
      return;
    if (!myIntRange.isEmpty())
      myNavigableSet.add(myIntRange);
  }

  public boolean isEmpty() {
    return myNavigableSet.isEmpty();
  }

  public Type getIntervalWith(int value) {
    for (Type myType : this)
      if (myType.contains(value))
        return myType;
    return null;
  }

  // public boolean isMember(int value) {
  // myNavigableSet.lower(value);
  // }
  //
  // public void removeInt(int value) {
  // IntRanges<IntRange> myIntRanges = new IntRanges<>();
  // myIntRanges.getCover();
  // }
  public void intersect(IntRange myIntRange) {
    intersect(myIntRange.min, myIntRange.max);
  }

  public void intersect(int min, int max) {
    Collection<Type> myList = new LinkedList<>(myNavigableSet);
    myNavigableSet.clear();
    for (Type myType : myList)
      union(myType.intersect(min, max));
  }

  public <More extends IntRange> void intersect(IntRanges<More> myIntRanges) {
    Collection<Type> myList = new LinkedList<>(myNavigableSet);
    myNavigableSet.clear();
    for (Type myType : myList)
      for (More myPart : myIntRanges)
        union(myType.intersect(myPart)); // inefficient algorithm
  }

  public int min() {
    return myNavigableSet.first().min;
  }

  public int max() {
    return myNavigableSet.last().max;
  }

  public IntRange getCover() {
    return new IntRange(min(), max());
  }

  public Stream<Integer> toIntStream() {
    return myNavigableSet.stream() //
        .flatMap(myIntRange -> myIntRange.stream().boxed()); //
  }

  public List<Integer> toIntList() {
    return toIntStream().collect(Collectors.toList());
  }

  @Override
  public Iterator<Type> iterator() {
    return myNavigableSet.iterator();
  }

  public void print() {
    for (Type myType : myNavigableSet)
      System.out.print(myType);
    System.out.println();
  }

  public static void demo1() {
    IntRanges<IntRange> myIntRanges = new IntRanges<>();
    System.out.println(myIntRanges.isEmpty());
    myIntRanges.union(new IntRange(-2, 10));
    System.out.println(myIntRanges.isEmpty());
    myIntRanges.intersect(15, 21);
    System.out.println(myIntRanges.isEmpty());
    myIntRanges.union(new IntRange(-2, 10));
    myIntRanges.union(new IntRange(20, 25));
    myIntRanges.union(new IntRange(11, 20));
    myIntRanges.union(new IntRange(0, 0));
    myIntRanges.intersect(4, 16);
    myIntRanges.print();
    for (int myInt : myIntRanges.toIntList())
      System.out.println(myInt);
  }
}
