// code by jph
package sys.mat;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class TwoLists {
  /** @param mySub sorted(!) list of Comparable
   * @param mySuper sorted(!) list of Comparable
   * @return true, if mySub is subset of mySuper */
  public static <Type extends Comparable<Type>> boolean areSortedSublists(List<Type> mySub, List<Type> mySuper) {
    int index = 0;
    int count = 0;
    // assertListSorted(mySub);
    // assertListSorted(mySuper);
    while (index < mySuper.size() && count < mySub.size()) {
      if (mySuper.get(index).compareTo(mySub.get(count)) < 0)
        ++index;
      else //
        if (mySuper.get(index).equals(mySub.get(count))) {
        ++index;
        ++count;
      } else
        return false;
    }
    return count == mySub.size();
  }

  /** symmetric distance because max function is symmetric
   * 
   * @param mySub
   * @param mySuper
   * @return */
  public static int symmetricDistance(List<Integer> mySub, List<Integer> mySuper) {
    return Math.max(directionalDistance(mySub, mySuper), directionalDistance(mySuper, mySub));
  }

  /** symmetric pseudo distance because min function is symmetric
   * 
   * @param mySub
   * @param mySuper
   * @return */
  public static int directionalDistanceMin(List<Integer> mySub, List<Integer> mySuper) {
    return Math.min(directionalDistance(mySub, mySuper), directionalDistance(mySuper, mySub));
  }

  public static int directionalDistance(List<Integer> mySub, List<Integer> mySuper) {
    int max = 0;
    if (!mySuper.isEmpty())
      for (int myInt : mySub) {
        int min = Math.abs(myInt - mySuper.get(0));
        for (int myCmp : mySuper.subList(1, mySuper.size()))
          min = Math.min(min, Math.abs(myInt - myCmp));
        max = Math.max(min, max);
      }
    return max;
  }

  @Deprecated
  static <Type extends Comparable<Type>> boolean assertListSorted(List<Type> myList) {
    // function can be private but is not used anyways
    for (int c0 = 1; c0 < myList.size(); ++c0)
      if (0 < myList.get(c0 - 1).compareTo(myList.get(c0)))
        new RuntimeException("not sorted " + myList);
    return true;
  }

  public static void main(String[] args) {
    List<Integer> ante = Arrays.asList(0, 4, 0);
    Collections.sort(ante);
    List<Integer> post = Arrays.asList(1, 10);
    Collections.sort(post);
    // System.out.println(TwoLists.areSortedSublists(ante, post));
    System.out.println("symm " + TwoLists.symmetricDistance(ante, post));
    System.out.println("dirm " + TwoLists.directionalDistanceMin(ante, post));
    System.out.println("a->p " + TwoLists.directionalDistance(ante, post));
    System.out.println("p->a " + TwoLists.directionalDistance(post, ante));
  }
}
