// code by jph
package sys.mat.num;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import sys.mat.BinomialCoefficient;

public class BerzierApproximation {
  private final List<Double> myList;
  public final int degree;
  private final List<Integer> myNoverK;

  public BerzierApproximation(List<Double> myList) {
    this.myList = myList;
    degree = myList.size() - 1;
    myNoverK = BinomialCoefficient.row(degree);
  }

  /** @param x is in the interval 0 and 1
   * @return */
  public double at(double x) {
    Iterator<Double> myIteratorP = myList.iterator();
    Iterator<Integer> myIteratorB = myNoverK.iterator();
    double sum = 0;
    for (int k = 0; k <= degree; ++k)
      sum += myIteratorP.next() * myIteratorB.next() * Math.pow(x, k) * Math.pow(1 - x, degree - k); // numerically most accurate but not fastest
    return sum;
  }

  /** evaluation of Bezier function at equi-distant samples over the interval [0, 1]
   * 
   * @param myList
   * @param samples
   * @return */
  public static List<Double> curve(List<Double> myList, int samples) {
    List<Double> myCurve = new ArrayList<>();
    BerzierApproximation myBerzierApproximation = new BerzierApproximation(myList);
    double fac = 1 / (double) (samples - 1);
    for (int count = 0; count < samples; ++count)
      myCurve.add(myBerzierApproximation.at(count * fac));
    return myCurve;
  }
}
