// code by Robert Sedgewick and Kevin Wayne
// adapted by jph
package sys.mat.num;

import java.awt.geom.Point2D;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

public class ConvexHull {
  /** @param myList
   * 
   * @param myList is permuted during the execution
   * @return points in clockwise (!) order with no 3 co-linear points */
  public static Stack<Point2D> grahamScan(List<Point2D> myList) {
    Comparator<Point2D> myComparator = new Comparator<Point2D>() {
      @Override
      public int compare(Point2D myP1, Point2D myP2) {
        int cmp = Double.compare(myP1.getY(), myP2.getY());
        return cmp != 0 ? cmp : Double.compare(myP1.getX(), myP2.getX());
      }
    };
    // option 1: sort
    // Collections.sort(myList, myComparator);
    // Point2D myPoint0 = myList.get(0);
    // option 2: min
    Point2D myPoint0 = Collections.min(myList, myComparator);
    Collections.sort(myList, new Comparator<Point2D>() {
      @Override
      public int compare(Point2D myP1, Point2D myP2) {
        int cmp = Double.compare(angle(myPoint0, myP1), angle(myPoint0, myP2));
        return cmp != 0 ? cmp : Double.compare(myPoint0.distanceSq(myP1), myPoint0.distanceSq(myP2));
      }
    });
    Stack<Point2D> myStack = new Stack<Point2D>();
    myStack.push(myPoint0);
    int k1 = 1;
    Point2D myPoint1 = null;
    for (Point2D myPoint2D : myList.subList(k1, myList.size())) {
      if (!myPoint0.equals(myPoint2D)) {
        myPoint1 = myPoint2D;
        break;
      }
      ++k1;
    }
    if (myPoint1 == null)
      return myStack;
    // System.out.println("seed1 " + k1);
    int k2 = k1 + 1;
    for (Point2D myPoint2D : myList.subList(k2, myList.size()))
      if (ccw(myPoint0, myPoint1, myPoint2D) == 0) {
        // System.out.println("co-lin");
        ++k2;
      } else
        break;
    // System.out.println("seed2 " + k2);
    // ---
    myStack.push(myList.get(k2 - 1));
    for (Point2D myPoint2D : myList.subList(k2, myList.size())) {
      Point2D top = myStack.pop();
      while (ccw(myStack.peek(), top, myPoint2D) <= 0)
        top = myStack.pop();
      myStack.push(top);
      myStack.push(myPoint2D);
    }
    return myStack;
  }

  public static double angle(Point2D myP1, Point2D myP2) {
    return Math.atan2(myP2.getY() - myP1.getY(), myP2.getX() - myP1.getX());
  }

  /** Three points are a counter-clockwise turn if ccw > 0, clockwise if
   * ccw < 0, and collinear if ccw = 0 because ccw is a determinant that
   * gives twice the signed area of the triangle formed by p1, p2 and p3.
   * (from Wikipedia)
   * 
   * @param myP1
   * @param myP2
   * @param myP3
   * @return */
  public static double ccw(Point2D myP1, Point2D myP2, Point2D myP3) {
    return (myP2.getX() - myP1.getX()) * (myP3.getY() - myP1.getY()) - (myP2.getY() - myP1.getY()) * (myP3.getX() - myP1.getX());
  }

  static void demo(List<Point2D> myList) {
    for (Point2D myPoint2D : myList)
      System.out.println(myPoint2D);
    System.out.println("---");
    for (Point2D myPoint2D : grahamScan(myList))
      System.out.println(myPoint2D);
  }

  // test client
  public static void main(String[] args) {
    List<Point2D> myList = new LinkedList<>();
    myList.add(new Point2D.Double(-1, 2));
    myList.add(new Point2D.Double(1, -1));
    myList.add(new Point2D.Double(3, -1));
    myList.add(new Point2D.Double(1, 1));
    myList.add(new Point2D.Double(2, 2));
    myList.add(new Point2D.Double(2, 2));
    myList.add(new Point2D.Double(2, 2));
    myList.add(new Point2D.Double(0, -1));
    myList.add(new Point2D.Double(0, -1));
    myList.add(new Point2D.Double(2, -1));
    myList.add(new Point2D.Double(2, -1));
    demo(myList);
  }
}
