// code by jph
package sys.mat.num;

import java.util.Arrays;
import java.util.List;

public abstract class ShepardInterpolation<Type> {
  public double threshold = 1e-6;
  public double exponent = 2;

  public abstract double distance(Type myT1, Type myT2);

  public double[] weights(List<Type> myList, Type myRef) {
    double[] myDouble = new double[myList.size()];
    double sum = 0;
    int count = 0;
    for (Type myType : myList) {
      double dist = distance(myType, myRef);
      if (dist < threshold) {
        for (int c0 = 0; c0 < myDouble.length; ++c0)
          myDouble[c0] = c0 == count ? 1 : 0;
        return myDouble;
      }
      myDouble[count] = 1 / Math.pow(dist, exponent);
      sum += myDouble[count];
      ++count;
    }
    for (int c0 = 0; c0 < myDouble.length; ++c0)
      myDouble[c0] /= sum; // prevents div by zero if list is empty
    return myDouble;
  }

  public static void main(String[] args) {
    ShepardInterpolation<Double> myShepardWeight = new ShepardInterpolation<Double>() {
      @Override
      public double distance(Double myT1, Double myT2) {
        return Math.abs(myT1 - myT2);
      }
    };
    List<Double> myL = Arrays.asList(new Double[] { 0., 1., 2., 3. });
    double[] myDouble = myShepardWeight.weights(myL, 2.000001);
    for (int c0 = 0; c0 < myDouble.length; ++c0) {
      System.out.println(myDouble[c0]);
    }
  }
}
