// code by jph
package sys.wav.get;

import javax.sound.sampled.TargetDataLine;

import sys.wav.MixerDataFormat;
import sys.wav.WaveIn;

/** TargetDataLineProvider represents a hardware based audio source available on the host system.
 * Examples are line-in channel, microphone, virtual audio cable, etc. */
public class TargetDataLineProvider extends AudioProviderThread {
  final MixerDataFormat myMixerDataFormat;
  private TargetDataLineSpy myTargetDataLineSpy;

  /** @param myMixerDataFormat points to hardware based audio source available on the host system */
  public TargetDataLineProvider(MixerDataFormat myMixerDataFormat) {
    this.myMixerDataFormat = myMixerDataFormat;
  }

  @Override
  public void beforeOpenProvider() {
    TargetDataLine myTargetDataLine = WaveIn.instance.lineFrom(myMixerDataFormat);
    myTargetDataLineSpy = new TargetDataLineSpy(myTargetDataLine);
    myTargetDataLineSpy.getBufferSize();
    myTargetDataLineSpy.open(myMixerDataFormat.myAudioFormat, (int) myMixerDataFormat.myAudioFormat.getSampleRate());
    myTargetDataLineSpy.start();
    myTargetDataLineSpy.getFormat();
  }

  @Override
  public void prepareBlocking(byte[] myByte) {
    myTargetDataLineSpy.read(myByte, 0, 512); // this might throw an exception when waiting for data and line is closed
  }

  @Override
  public void afterProviderClosed() {
    myTargetDataLineSpy.close();
  }

  @Override
  public String toString() {
    return myMixerDataFormat.toString();
  }
}
